package com.zulutrade.sample.trading.client.trading;

import org.codehaus.jackson.map.ObjectMapper;

import com.zulutrade.sample.trading.client.configuration.ZuluTradeAuthenticationInfo;
import com.zulutrade.sample.trading.client.configuration.ZuluTradingServerConfig;
import com.zulutrade.sample.trading.client.sender.TradingResquestsSender;
import com.zulutrade.sample.trading.client.trading.dtos.ClosedTradesResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.NewMarketTradeResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.NewOrderResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.OpenTradesResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.OrderCanceledResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.ProviderStatisticsResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.TradeClosedResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.TradingConfigurationResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.UpdateTrailingStopValueResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.UpdateValueResponseMessage;


public class ZuluTradingGatewayImpl implements ZuluTradingGateway {

	private final ZuluTradingRequestGenerator generator = new ZuluTradingRequestGeneratorImpl();
	private final ObjectMapper objectMapper = new ObjectMapper();
	private final ZuluTradeAuthenticationInfo authenticationInfo;
	private final ZuluTradingServerConfig config;
	
	public ZuluTradingGatewayImpl(ZuluTradeAuthenticationInfo authenticationInfo, ZuluTradingServerConfig config){
		this.authenticationInfo = authenticationInfo;
		this.config = config;
	}
		
	public NewMarketTradeResponseMessage openMarket(String currencyName, double lots, boolean buy, double requestedPrice, String uniqueId) throws Exception {		
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getOpenMarketRequest(currencyName, lots, buy, requestedPrice, uniqueId));		
		try{
			return objectMapper.readValue(response, NewMarketTradeResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}

	public NewOrderResponseMessage openOrder(String currencyName, double lots, boolean buy, double requestedPrice, String uniqueId) throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getOpenOrderRequest(currencyName, lots, buy, requestedPrice, uniqueId));		
		try{
			return objectMapper.readValue(response, NewOrderResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}

	public OrderCanceledResponseMessage cancelOrder(String currencyName, double lots, boolean buy, String uniqueId) throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getCancelOrderRequest(currencyName, lots, buy, uniqueId));		
		try{
			return objectMapper.readValue(response, OrderCanceledResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}


	public TradeClosedResponseMessage closeMarket(String currencyName, double lots, boolean buy, String uniqueId, double requestedPrice) throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getCloseMarketRequest(currencyName, lots, buy, requestedPrice, uniqueId));		
		try{
			return objectMapper.readValue(response, TradeClosedResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}


	public UpdateValueResponseMessage updateStop(String currencyName, boolean buy, double lots, String uniqueId, double stopValue) throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getUpdateStopRequest(currencyName, buy, lots, uniqueId, stopValue));		
		try{
			return objectMapper.readValue(response, UpdateValueResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}
	
	@Override
	public UpdateTrailingStopValueResponseMessage updateTrailingStop(String currencyName, boolean buy, double lots, String uniqueId, int trailingStopValue) throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getUpdateTrailingStopRequest(currencyName, buy, lots, uniqueId, trailingStopValue));
		try{
			return objectMapper.readValue(response, UpdateTrailingStopValueResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}
	
	@Override
	public UpdateTrailingStopValueResponseMessage updateTrailingStopCondition(String currencyName, boolean buy, double lots,
			String uniqueId, int trailingStopValueToSet, int tralingStopConditionPipsToSet) throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getUpdateTrailingStopConditionRequest(currencyName, buy, lots, uniqueId, trailingStopValueToSet, tralingStopConditionPipsToSet));
		try{
			return objectMapper.readValue(response, UpdateTrailingStopValueResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}


	public UpdateValueResponseMessage updateLimit(String currencyName, boolean buy, double lots, String uniqueId,
			double limitValue) throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getUpdateLimitRequest(currencyName, buy, lots, uniqueId, limitValue));		
		try{
			return objectMapper.readValue(response, UpdateValueResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}

	
	public UpdateValueResponseMessage updateEntryRate(String currencyName, boolean buy, double lots, String uniqueId,
			double entryValue) throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getUpdateEntryRateRequest(currencyName, buy, lots, uniqueId, entryValue));		
		try{
			return objectMapper.readValue(response, UpdateValueResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}

	public OpenTradesResponseMessage getOpenTrades() throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getGetOpenTradesRequest());
		try{
			return objectMapper.readValue(response, OpenTradesResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}

	@Override
	public TradingConfigurationResponseMessage getTradingConfiguration() throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getTradingConfigurationRequest());
		try{
			return objectMapper.readValue(response, TradingConfigurationResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}

	@Override
	public ProviderStatisticsResponseMessage getProviderStatistics() throws Exception {
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getProviderStatisticsRequest());
		try{
			return objectMapper.readValue(response, ProviderStatisticsResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}

	@Override
	public ClosedTradesResponseMessage getHistory(String startDate, String endDate) throws Exception{
		String response = TradingResquestsSender.sendRequest(config, authenticationInfo, generator.getHistoryRequest(startDate, endDate));
		try{
			return objectMapper.readValue(response, ClosedTradesResponseMessage.class);
		}catch(Exception ex){
			throw new Exception("Server response: "+response);
		}
	}
}
